// global arrays
var apps = [];
var cats = [];

function MainAssistant()
{
	// main list model
	this.mainModel = 
	{
		items: [
		{
			name: $L('Check for Updates'),	// displays in list
			style: 'disabled',				// class for use in the list display
			scene: 'app-list',				// scene that will be pushed on tap 
			list: 'updates'					// variable used by the app-list scene to display the correct list other scenes will have different variables
		},
		{
			name: $L('WebOS Applications'),
			style: false,
			scene: 'app-categories'
		},
		{
			name: $L('WebOS Patches'),
			style: 'disabled',
			scene: false
		},
		{
			name: $L('List of Installed Items'),
			style: 'disabled',
			scene: 'app-list',
			list: 'installed'
		},
		{
			name: $L('Alpha List of Everything'),
			style: false,
			scene: 'app-list',
			list: 'all'
		}]
	};
	
	// spinner model
	this.spinnerModel = {spinning: true};
}

MainAssistant.prototype.setup = function()
{
	// setup spinner model
	this.controller.setupWidget('spinner', {spinnerSize: 'large'}, this.spinnerModel);
	
	// setup list model
	this.controller.setupWidget('mainList', { itemTemplate: "main/rowTemplate", swipeToDelete: false, reorderable: false }, this.mainModel);
	Mojo.Event.listen(this.controller.get('mainList'), Mojo.Event.listTap, this.listTapHandler.bindAsEventListener(this));
	
	// hide the list while we update ipkg
	this.controller.get('mainList').style.display = "none";
	
	// start with a test for an internet connection
	this.controller.get('spinnerStatus').innerHTML = "Checking Connection";
	this.controller.serviceRequest('palm://com.palm.connectionmanager', {
	    method: 'getstatus',
	    onSuccess: this.onConnection.bindAsEventListener(this),
	    onFailure: this.onConnection.bindAsEventListener(this)
	});
}

MainAssistant.prototype.listTapHandler = function(event)
{
	if (event.item.scene === false || event.item.style == 'disabled') 
	{
		// no scene or its disabled, so we won't do anything
	}
	else
	{
		// push the scene
		this.controller.stageController.pushScene(event.item.scene, event.item);
	}
}

MainAssistant.prototype.onConnection = function(response)
{
	if (response && response.returnValue === true && response.isInternetConnectionAvailable === true)
	{
		// initiate update if we have a connection
		this.controller.get('spinnerStatus').innerHTML = "Updating";
		IPKGService.update(this.onUpdate.bindAsEventListener(this));
	}
	else
	{
		// if not, go right to loading the app info
		this.controller.get('spinnerStatus').innerHTML = "Loading";
		IPKGService.info(this.onInfo.bindAsEventListener(this));
	}
}

MainAssistant.prototype.onUpdate = function(payload)
{
	if (!payload) 
	{
		// if its not running, it actually never gets here, as it never returns
		// the command below will start the service
		// luna-send -n 1 palm://org.webosinternals.ipkgservice/list {}
		Mojo.Controller.errorDialog('Update Error. The service probably isn\'t running.');
		this.hideSpinner();
	}
	else if (payload.returnVal != undefined) 
	{
		// its returned, but we don't really care if anything was actually updated
		//console.log(payload.returnVal);
		
		// lets call the function to update the global list of apps
		this.controller.get('spinnerStatus').innerHTML = "Loading";
		IPKGService.info(this.onInfo.bindAsEventListener(this));
	}
}

MainAssistant.prototype.onInfo = function(payload)
{
	if (!payload) 
	{
		Mojo.Controller.errorDialog('Unable to get list of apps.');
		this.hideSpinner();
	}
	else
	{
		for (var x = 0; x < payload.info.length; x++)
		{
			// check if Source is json object
			// basically, if it has a { in it, we'll assume its json data
			if (payload.info[x].Source != undefined && payload.info[x].Source.indexOf('{') > -1) 
			{
				payload.info[x].SourceObj = JSON.parse(payload.info[x].Source);
			}
			
			var appNum = this.appInList(payload.info[x].Package);
			if (appNum === false) 
			{
				// add install variable
				if (payload.info[x].Status.indexOf('not-installed') > -1) 
				{
					payload.info[x].Installed = false;
				}
				else 
				{
					payload.info[x].Installed = true;
					// since something is installed, we should enable the installed list
					this.mainModel.items[3].style = false;
				}
				
				// add default no-update
				payload.info[x].Update = false;
				
				// add this package to global app list
				apps.push(payload.info[x]);
				
				// add this packages category to global category list
				var catNum = this.catInList(payload.info[x].Section);
				if (catNum === false) 
				{
					// push new category
					cats.push({name: payload.info[x].Section, count: 1});
				}
				else
				{
					// increment category count
					cats[catNum].count++;
				}
			}
			else
			{				
				// check if its newer
				var newer = this.versionNewer(apps[appNum].Version, payload.info[x].Version);
				
				// if they're both not installed, and this is newer, replace the old one
				if (payload.info[x].Status.indexOf('not-installed') > -1 &&
					apps[appNum].Status.indexOf('not-installed') > -1 &&
					newer)
				{
					apps[appNum] = payload.info[x];
					continue;
				}
				
				// if the new one is not installed and the old one is, and its older, update the old one
				if (payload.info[x].Status.indexOf('not-installed') == -1 &&
					apps[appNum].Status.indexOf('not-installed') > -1 &&
					!newer)
				{
					apps[appNum].Installed = true;
					apps[appNum].Update = true;
					apps[appNum].VersionOld = payload.info[x].Version;
					// since we've found a package that needs an update, we'll enable the updates item
					this.mainModel.items[0].style = false;
					continue;
				}
				
				// if the new one is not installed but the old one is, and this is newer, replace the old one
				if (payload.info[x].Status.indexOf('not-installed') == -1 &&
					apps[appNum].Status.indexOf('not-installed') > -1 &&
					newer)
				{
					payload.info[x].Installed = true;
					payload.info[x].Update = false;
					apps[appNum] = payload.info[x];
					continue;
				}
			}
		}
		
		// sort the packages and categories
		apps.sort(function(a, b)
		{
			if (a.Description && b.Description) return ((a.Description.toLowerCase() < b.Description.toLowerCase()) ? -1 : ((a.Description.toLowerCase() > b.Description.toLowerCase()) ? 1 : 0));
			else return -1;
		});
		cats.sort(function(a, b)
		{
			if (a.name && b.name) return ((a.name < b.name) ? -1 : ((a.name > b.name) ? 1 : 0));
			else return -1;
		});
		
		// update the list because we may have found something to update
		this.controller.get('mainList').mojo.noticeUpdatedItems(0, this.mainModel.items);
		
		// hide the spinner
		this.hideSpinner();
	}
}

// determines if one version number is newer then the other.
MainAssistant.prototype.versionNewer = function(one, two)
{
	// if one >= two returns false
	// if one < two returns true
	var v1 = one.split('.');
	var v2 = two.split('.');
	if (parseInt(v2[0]) > parseInt(v1[0])) return true; 
	else if (parseInt(v2[0]) == parseInt(v1[0]) && parseInt(v2[1]) > parseInt(v1[1])) return true;
	else if (parseInt(v2[0]) == parseInt(v1[0]) && parseInt(v2[1]) == parseInt(v1[1]) && parseInt(v2[2]) > parseInt(v1[2])) return true;
	return false;
}

// for checking if a package is already in the global list
MainAssistant.prototype.appInList = function(pkg)
{
	if (apps.length > 0)
	{
		for (var a = 0; a < apps.length; a++)
		{
			if (apps[a].Package == pkg) 
			{
				return a;
			}
		}
	}
	return false;
}

// for checking if a category is already in the global list
MainAssistant.prototype.catInList = function(cat)
{
	if (cats.length > 0) 
	{
		for (var c = 0; c < cats.length; c++) 
		{
			if (cats[c].name == cat) 
			{
				return c;
			}
		}
	}
	return false;
}

MainAssistant.prototype.hideSpinner = function()
{
	// stop and hide the spinner
	this.spinnerModel.spinning = false;
	this.controller.modelChanged(this.spinnerModel);
	
	// show the list
	this.controller.get('mainList').style.display = "inline";
}

MainAssistant.prototype.activate = function(event) {}

MainAssistant.prototype.deactivate = function(event) {}

MainAssistant.prototype.cleanup = function(event) {}
