#!/bin/sh
#
# List of Optware applications to start/stop
# in conjunction with mounting/unmounting VFS.
# OpenSSH and Dropbear are most common.
# Edit to suit environment.
APPS="optware-openssh optware-dropbear"
 
# An entry in /etc/fstab must exist for the VFS
# Grab and parse entry
FSVFS=`/bin/grep "/opt" /etc/fstab|grep loop`
if [ "$FSVFS" != "" ]
then
 # Complete FileName of VFS to mount/unmount
 LVFSFN=`echo $FSVFS|/usr/bin/awk '{print $1}'`
 # VFS Mount Point (ie /opt)
 LVFSMP=`echo $FSVFS|/usr/bin/awk '{print $2}'`
 # VFS file system Type (ie ext3)
 LVFSTP=`echo $FSVFS|/usr/bin/awk '{print $3}'`
else
 echo "**** No /etc/fstab entry"
 exit 1
fi
 
# Determine if VFS is mounted
VFSMTD=`/bin/df|/bin/grep "$LVFSMP"|/bin/grep "^/dev/loop"`
if [ "$VFSMTD" != "" ]
then
 # Loopback device associated with VFS (ie /dev/loop0)
 LVFSDV=`echo $VFSMTD|/usr/bin/awk '{print $1}'`
fi
 
case $1 in
  "on" )
        if [ -e "$LVFSFN" -a -d "$LVFSMP" -a "$VFSMTD" = "" ]
        then
         echo '**** Mounting Linux VFS'
         /bin/mount "$LVFSFN"
         echo '**** Starting Applications'
         for x in $APPS
           do
            if [ -f /etc/event.d/"$x" ]
            then
            /sbin/initctl start "$x"
            fi
         done
         echo '**** Linux VFS Enabled'
        else
         echo '**** Error mounting VFS'
         exit 1
        fi
        ;;
  "off" )
        if [ "$VFSMTD" != "" -a "$LVFSDV" != "" ]
        then
         echo '**** Stopping Applications'
         for x in $APPS
           do
            if [ -f /etc/event.d/"$x" ]
            then
            /sbin/initctl stop "$x"
            fi
         done
         echo '**** Unmounting Linux VFS'
         /bin/umount "$LVFSMP"
         # Workaround to disassociate VFS from loopback device
         /sbin/losetup -d $LVFSDV
         echo '**** Linux VFS Disabled'
        else
         echo '**** Error unmounting VFS'
         exit 1
        fi
        ;;
  * )
        echo "Usage: `/usr/bin/basename $0` on|off"
        ;;
esac